import { Injectable, NgModule, ModuleWithProviders } from '@angular/core';
import { ValidatorFn, FormBuilder, FormGroup } from '@angular/forms';

export class FormControl {
  controlName: string;
  model: any;
  validator?: ValidatorFn | ValidatorFn[];
  error?: any;
}

@Injectable()
export class ReactiveFormsService {

  constructor(
    private fb: FormBuilder,
  ) { }

  public getFormGroup(formGroup: Array<FormControl>) {
    let form = this.buildForm(formGroup);
    form.valueChanges.subscribe(changeValue => {
      this.onValueChanged(changeValue, formGroup, form);
    });
    this.onValueChanged();
    return form;
  }

  //构建表单
  private buildForm(formGroup: Array<FormControl>): FormGroup {
    let form: any = {};
    formGroup.forEach((formControl) => {
      form[formControl.controlName] = [formControl.model, formControl.validator];
    });
    return this.fb.group(form);
  }

  //表单变动事件
  private onValueChanged(changeValue?: any, formGroup?: Array<FormControl>, form?: FormGroup) {
    if (!formGroup) return;
    for (let control of formGroup) {
      control['errorInfo'] = '';
      let formControl = form.get(control.controlName);
      if (formControl && formControl.dirty && !formControl.valid) {
        for (let key in formControl.errors) {
          control['errorInfo'] += control.error[key] + ' ';
        }
      }
    }
  }

}

@NgModule({
  imports: [],
  exports: [],  
})
export class ReactiveFormsModule {
  static forRoot(): ModuleWithProviders {
    return {
      ngModule: ReactiveFormsModule,
      providers: [ReactiveFormsService]
    };
  }
}